/*
  Display.cpp - Library to control a ST7565 based 12864 LCD.
  Thomas Baum, th.baum@online.de, 10.05.2014
*/

#include <SPI.h>
#include "Display.h"

const int LCD_A0 = 8;
const int LCD_RST = 9;
const int LCD_CS = 10;

void Display::init(byte contrast) {
  SPI.begin();
  SPI.setBitOrder(MSBFIRST);
  pinMode(LCD_A0, OUTPUT);
  pinMode(LCD_RST, OUTPUT);
  pinMode(LCD_CS, OUTPUT);

  digitalWrite(LCD_RST, LOW);
  digitalWrite(LCD_CS, LOW);
  delay(50);
  digitalWrite(LCD_RST, HIGH);

  writeCommand(0xa0);
  writeCommand(0xc8);
  writeCommand(0xa6);
  writeCommand(0xa2);
  writeCommand(0x2f);
  writeCommand(0xf8);
  writeCommand(0x00);
  writeCommand(0x27);
  writeCommand(0x81);
  writeCommand(contrast);
  writeCommand(0xac);
  writeCommand(0x00);
  writeCommand(0xa4);
  writeCommand(0xaf);
}

void Display::writeCommand(byte command) {
  digitalWrite(LCD_A0, LOW);
  SPI.transfer(command);
  digitalWrite(LCD_A0, HIGH);
}

void Display::writeData(byte data) {
  SPI.transfer(data);
}

void Display::resetRamAddress(void) {
  writeCommand(0x10);
  writeCommand(0x00);
  writeCommand(0xb0);
}

void Display::setPageAddress(byte pageAddress) {
         writeCommand((byte) pageAddress | 0xb0);
}

void Display::setColumnAddress(byte columnAddress) {
         byte highNibble = (byte)columnAddress >> 4;
         writeCommand(0x10 | highNibble);
         byte lowNibble = (byte)columnAddress & 0x0f;
	 writeCommand(lowNibble);
}

void Display::clearDisplayRAM() {
         for(int i = 0; i < 8; i++) {
            setPageAddress(i);
            setColumnAddress(0);
            for(int j = 0; j < 128; j++) {
                writeData(0x00);
            }
         }
}


